@@ -2,7 +2,7 @@ from driftpy.constants.numeric_constants import *
 from driftpy.types import *
 from driftpy.accounts import *
 from driftpy.math.oracle import OracleData
-from driftpy.math.spot_market import get_token_value
+from driftpy.math.spot_market import *
 
 from enum import Enum
 
@@ -86,8 +86,72 @@ def calculate_size_premium_liability_weight(
     max_liability_weight = max(liability_weight, size_premium_liability_weight)
     return max_liability_weight
 
+def calculate_net_user_pnl(
+    perp_market: PerpMarket, 
+    oracle_data: OracleData
+):
+    net_user_position_value = (
+        perp_market.amm.base_asset_amount_with_amm
+        * oracle_data.price
+        / BASE_PRECISION
+        / PRICE_TO_QUOTE_PRECISION_RATIO
+    )
 
-def calculcate_liability_weight(
+    net_user_cost_basis = perp_market.amm.quote_asset_amount
+    net_user_pnl = net_user_position_value + net_user_cost_basis
+
+    return net_user_pnl
+
+def calculate_net_user_pnl_imbalance(
+    perp_market: PerpMarket, 
+    spot_market: SpotMarket, 
+    oracle_data: OracleData
+):
+    user_pnl = calculate_net_user_pnl(
+        perp_market, oracle_data
+    )
+
+    pnl_pool = get_token_amount(
+        perp_market.pnl_pool.scaled_balance, 
+        spot_market, 
+        "SpotBalanceType.Deposit()"
+    )
+
+    imbalance = user_pnl - pnl_pool
+    return imbalance
+
+def calculate_unrealized_asset_weight(
+    perp_market: PerpMarket, 
+    spot_market: SpotMarket, 
+    unrealized_pnl: int, 
+    margin_category: MarginCategory, 
+    oracle_data: OracleData
+):
+    match margin_category:
+        case MarginCategory.INITIAL:
+            asset_weight = perp_market.unrealized_pnl_initial_asset_weight
+            if perp_market.unrealized_pnl_max_imbalance > 0: 
+                net_unsettled_pnl = calculate_net_user_pnl_imbalance(
+                    perp_market, 
+                    spot_market, 
+                    oracle_data
+                )
+                if net_unsettled_pnl > perp_market.unrealized_pnl_max_imbalance:
+                    asset_weight = asset_weight * perp_market.unrealized_pnl_max_imbalance / net_unsettled_pnl
+                
+            asset_weight = calculate_size_discount_asset_weight(
+                unrealized_pnl, 
+                perp_market.unrealized_pnl_imf_factor, 
+                asset_weight
+            )
+        case MarginCategory.MAINTENANCE:
+            asset_weight = perp_market.unrealized_pnl_maintenance_asset_weight
+        case _: 
+            raise Exception(f"invalid margin category: {margin_category}")
+
+    return asset_weight
+
+def calculate_liability_weight(
     balance_amount: int, spot_market: SpotMarket, margin_category: MarginCategory
 ) -> int:
     size_precision = 10 ** spot_market.decimals
@@ -104,14 +168,14 @@ def calculcate_liability_weight(
         case MarginCategory.INITIAL:
             asset_weight = calculate_size_premium_liability_weight(
                 size_in_amm_reserve_precision,
-                spot_market.imf_factorm,
+                spot_market.imf_factor,
                 spot_market.initial_liability_weight,
                 SPOT_WEIGHT_PRECISION,
             )
         case MarginCategory.MAINTENANCE:
             asset_weight = calculate_size_premium_liability_weight(
                 size_in_amm_reserve_precision,
-                spot_market.imf_factorm,
+                spot_market.imf_factor,
                 spot_market.maintenance_liability_weight,
                 SPOT_WEIGHT_PRECISION,
             )
@@ -162,7 +226,7 @@ def get_spot_liability_value(
     liability_value = get_token_value(token_amount, spot_market.decimals, oracle_data)
 
     if margin_category is not None:
-        weight = calculcate_liability_weight(token_amount, spot_market, margin_category)
+        weight = calculate_liability_weight(token_amount, spot_market, margin_category)
 
         if margin_category == MarginCategory.INITIAL:
             assert max_margin_ratio, "set = user.max_margin_ratio"
